#pragma once

#include "StateVector.h"
#include "TM.h"
#include "ephi.h"
#include "DVectors.h"
#include "ires.h"

class XInterpolator
{
public:
    XInterpolator();
    XInterpolator(const SInitOrbit& orbit, const TimeJD& beg, const TimeJD& end, bool calc_rnb);
    XInterpolator(
        const std::vector<IRes>& vec,
        const std::vector<std::tuple<double, double, double>>& rnb);
    Vect6 get_pos(const TimeJD& date, bool tks = false) const;
    TimeJD get_beg() const { return m_beg; }
    TimeJD get_end() const { return m_end; }
    static void calc_positions(
        const SInitOrbit& orbit, 
        const TimeJD& beg, 
        const TimeJD& end,
        bool calc_rnb,
        std::vector<IRes>& xyz,
        std::vector<std::tuple<double, double, double>>& rnb);
    void export_data() const;
    std::vector<IRes> m_positions;
    std::vector<std::tuple<double, double, double>> m_rnb;
private:
    void create(const SInitOrbit& orbit, const TimeJD& beg, const TimeJD& end, bool calc_rnb);
    void create(
        const std::vector<IRes>& xyz,
        const std::vector<std::tuple<double, double, double>>& rnb);
    static void propagate(
        const SInitOrbit& orbit,
        const TimeJD& beg,
        const TimeJD& end,
        int dir,
        const bool calc_rnb,
        std::vector<IRes>& xyz,
        std::vector<std::tuple<double, double, double>>& rnb);
    static void propagate_dir(
        const SInitOrbit& orbit,
        const TimeJD& beg,
        const TimeJD& end,
        const int dir,
        bool calc_rnb,
        std::vector<IRes>& xyz,
        std::vector<std::tuple<double, double, double>>& rnb);
    TimeJD m_beg;
    TimeJD m_end;
    ei m_ephi;
    ei m_rnb_ephi;
};
